"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const lazyLoadedSymbol = Symbol('LAZY_LOADED');
exports.default = (lazyLoadMap) => {
    return async (resolve, node, args, context, info) => {
        const lazyLoad = lazyLoadMap[info.parentType.name];
        if (!lazyLoad) {
            return resolve(node, args, context, info);
        }
        const currentValue = node[info.fieldName];
        if (currentValue !== undefined) {
            return resolve(node, args, context, info);
        }
        if (!node[lazyLoadedSymbol]) {
            node[lazyLoadedSymbol] = lazyLoad(node, context);
        }
        const newNode = await node[lazyLoadedSymbol];
        return resolve(newNode, args, context, info);
    };
};
